"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resync = exports.except = exports.affect = exports.state = exports.wait = void 0;
const interrupt_1 = require("./interrupt");
const state_1 = require("./state");
const debug = require('debug')('mocko:resync');
let queue = [];
function wait(provider) {
    if (queue.length) {
        try {
            return queue.shift().run();
        }
        catch (e) {
            if (!(e instanceof interrupt_1.ReexecuteInterrupt)) {
                throw e;
            }
        }
    }
    throw new interrupt_1.AsyncInterrupt(provider());
}
exports.wait = wait;
function state(fn) {
    return wait(async () => fn());
}
exports.state = state;
function affect(fn) {
    state(fn);
}
exports.affect = affect;
function except(e) {
    if (e instanceof interrupt_1.AsyncInterrupt) {
        throw e;
    }
}
exports.except = except;
function resync(fn, { limit = 100 } = {}) {
    async function runEvaluation(state, ...params) {
        try {
            queue = [...state.queue];
            debug(`starting execution with ${queue.length} results: ` + queue.map(v => v.toString()).join(', '));
            const value = fn(...params);
            if (queue.length) {
                throw new Error('Resync evaluation differed between runs, make sure impure functions are called inside state()');
            }
            return value;
        }
        catch (e) {
            if (!(e instanceof interrupt_1.AsyncInterrupt)) {
                throw e;
            }
            await state.updateQueue(e.promise, limit);
            return await runEvaluation(state, ...params);
        }
    }
    return async (...params) => await runEvaluation(new state_1.ExecutionState(), ...params);
}
exports.resync = resync;
