"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecutionState = void 0;
const execution_1 = require("./execution");
const interrupt_1 = require("./interrupt");
class ExecutionState {
    constructor() {
        this.queue = [];
        this.retries = 0;
    }
    async updateQueue(promise, limit) {
        if (this.retries > limit) {
            throw new Error('Too many resynchronizations, limit: ' + limit);
        }
        this.retries++;
        const queueDeepness = this.queue.filter(e => e instanceof execution_1.UnknownExecution).length;
        const [deepness, execution] = await this.updateInstructions(promise);
        this.mergeInstructionsToQueue(queueDeepness, deepness, execution);
    }
    async updateInstructions(promise, deepness = 0) {
        try {
            const value = await promise;
            return [deepness, new execution_1.SuccessExecution(value)];
        }
        catch (e) {
            if (e instanceof interrupt_1.AsyncInterrupt) {
                return await this.updateInstructions(e.promise, deepness + 1);
            }
            else {
                return [deepness, new execution_1.FailureExecution(e)];
            }
        }
    }
    mergeInstructionsToQueue(queueDeepness, deepness, execution) {
        if (queueDeepness <= deepness) {
            const unks = new Array(deepness - queueDeepness)
                .fill(new execution_1.UnknownExecution());
            this.queue.push(...unks);
        }
        else {
            for (let diff = queueDeepness - deepness; diff > 0; diff--) {
                let exec;
                do {
                    exec = this.queue.pop();
                    if (typeof exec === 'undefined') {
                        throw new Error('Undefined execution');
                    }
                } while (!(exec instanceof execution_1.UnknownExecution));
            }
        }
        this.queue.push(execution);
    }
}
exports.ExecutionState = ExecutionState;
